@@ -1,31 +1,25 @@
 /*                     __                                               *\
 **     ________ ___   / /  ___     Scala API                            **
-**    / __/ __// _ | / /  / _ |    (c) 2003-2007, LAMP/EPFL             **
+**    / __/ __// _ | / /  / _ |    (c) 2003-2010, LAMP/EPFL             **
 **  __\ \/ /__/ __ |/ /__/ __ |    http://scala-lang.org/               **
 ** /____/\___/_/ |_/____/_/ | |                                         **
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: CountedIterator.scala 12652 2007-08-23 10:53:48Z michelou $
+// $Id: CountedIterator.scala 20028 2009-12-07 11:49:19Z cunei $
 
 
 package scala
 
 /** Counted iterators keep track of the number of elements seen so far
  *
- *  @author  Martin Odersky
- *  @version 1.0, 16/07/2003
+ *  @since 2.0
  */
+@deprecated("use iterator.zipWithIndex instead")
 trait CountedIterator[+A] extends Iterator[A] {
   /** counts the elements in this iterator; counts start at 0
    */
   def count: Int
   
   override def counted : this.type = this
-  override def buffered: BufferedIterator[A] with CountedIterator[A] = new BufferedIterator.Default[A] with CountedIterator[A] {
-    protected def fill = if (CountedIterator.this.hasNext) (CountedIterator.this.next) :: Nil else Nil
-    override def count = CountedIterator.this.count - peekList(0).length
-    override def counted : this.type = this
-    override def buffered : this.type = this
-  }
 }
