@@ -1,26 +1,27 @@
 /*                     __                                               *\
 **     ________ ___   / /  ___     Scala API                            **
-**    / __/ __// _ | / /  / _ |    (c) 2003-2009, LAMP/EPFL             **
-**  __\ \/ /__/ __ |/ /__/ __ |                                         **
+**    / __/ __// _ | / /  / _ |    (c) 2003-2010, LAMP/EPFL             **
+**  __\ \/ /__/ __ |/ /__/ __ |    http://scala-lang.org/               **
 ** /____/\___/_/ |_/____/_/ | |                                         **
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: Undoable.scala 16894 2009-01-13 13:09:41Z cunei $
+// $Id: Undoable.scala 21512 2010-04-12 21:53:30Z prokopec $
 
 
-package scala.collection.mutable
+package scala.collection
+package mutable
 
 
-/** Classes that mix in the <code>Undoable</code> class provide an operation
- *  <code>undo</code> which can be used to undo the last operation.
+/** Classes that mix in the `Undoable` class provide an operation
+ *  `undo` which can be used to undo the last operation.
  *
  *  @author  Matthias Zenger
  *  @version 1.0, 08/07/2003
+ *  @since   1
  */
 trait Undoable {
-
-    /** Undo the last operation.
-     */
-    def undo()
+  /** Undo the last operation.
+   */
+  def undo(): Unit
 }
