@@ -1,20 +1,20 @@
 /*                     __                                               *\
 **     ________ ___   / /  ___     Scala API                            **
-**    / __/ __// _ | / /  / _ |    (c) 2006-2007, LAMP/EPFL             **
+**    / __/ __// _ | / /  / _ |    (c) 2006-2010, LAMP/EPFL             **
 **  __\ \/ /__/ __ |/ /__/ __ |    http://scala-lang.org/               **
 ** /____/\___/_/ |_/____/_/ | |                                         **
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: Parsers.scala 12357 2007-07-18 21:55:08Z moors $
+// $Id: Parsers.scala 21343 2010-04-06 01:26:31Z extempore $
 
 package scala.util.parsing.combinator
 
 import scala.util.parsing.input._
-import scala.collection.mutable.{Map=>MutableMap}
+import scala.collection.mutable.ListBuffer
+import scala.annotation.tailrec
 
 // TODO: better error handling (labelling like parsec's <?>)
-// TODO: memoisation (like packrat parsers?) 
 
 /** <p>
  *    <code>Parsers</code> is a component that <i>provides</i> generic
@@ -48,35 +48,21 @@ import scala.collection.mutable.{Map=>MutableMap}
  *    of the input.
  *  </p>
  *
- * @requires Elem the type of elements the provided parsers consume 
- *              (When consuming invidual characters, a parser is typically called a ``scanner'', 
- *               which produces ``tokens'' that are consumed by what is normally called a ``parser''.
- *               Nonetheless, the same principles apply, regardless of the input type.)</p>
- *<p>
- * @provides Input = Reader[Elem] 
- *              The type of input the parsers in this component expect.</p>
- *<p>
- * @provides Parser[+T] extends (Input => ParseResult[T]) 
- *              Essentially, a `Parser[T]' is a function from `Input' to `ParseResult[T]'.</p>
- *<p>
- * @provides ParseResult[+T] is like an `Option[T]', in the sense that it is either
- *              `Success[T]', which consists of some result (:T) (and the rest of the input) or
- *              `Failure[T]', which provides an error message (and the rest of the input).</p>
- *
  * @author Martin Odersky, Iulian Dragos, Adriaan Moors 
  */
 trait Parsers {
-  /** the type of input elements */
+  /** the type of input elements the provided parsers consume (When consuming invidual characters, a parser is typically
+   * called a ``scanner'', which produces ``tokens'' that are consumed by what is normally called a ``parser''.
+   * Nonetheless, the same principles apply, regardless of the input type.) */
   type Elem
 
-  /** The parser input is an abstract reader of input elements */
+  /** The parser input is an abstract reader of input elements, i.e. the type of input the parsers in this component
+   * expect. */
   type Input = Reader[Elem]
 
-  /** A base class for parser results. 
-   *  A result is either successful or not (failure may be fatal, i.e.,
-   *  an Error, or not, i.e., a Failure)
-   *  On success, provides a result of type <code>T</code>. 
-   */
+  /** A base class for parser results. A result is either successful or not (failure may be fatal, i.e., an Error, or
+   * not, i.e., a Failure). On success, provides a result of type `T` which consists of some result (and the rest of
+   * the input). */
   sealed abstract class ParseResult[+T] {
     /** Functional composition of ParseResults
      * 
@@ -154,6 +140,15 @@ trait Parsers {
 
     def get: Nothing = error("No result when parsing failed")
   }
+  /** An extractor so NoSuccess(msg, next) can be used in matches.
+   */
+  object NoSuccess {
+    def unapply[T](x: ParseResult[T]) = x match {
+      case Failure(msg, next)   => Some(msg, next)
+      case Error(msg, next)     => Some(msg, next)
+      case _                    => None
+    }
+  }
   
   /** The failure case of ParseResult: contains an error-message and the remaining input.
    * Parsing will back-track when a failure occurs.
@@ -230,10 +225,28 @@ trait Parsers {
      */
     def ~ [U](p: => Parser[U]): Parser[~[T, U]] = (for(a <- this; b <- p) yield new ~(a,b)).named("~")
 
+    /** A parser combinator for sequential composition which keeps only the right result 
+     *
+     * <p> `p ~> q' succeeds if `p' succeeds and `q' succeeds on the input
+     *           left over by `p'.</p>
+     * 
+     * @param q a parser that will be executed after `p' (this parser) succeeds
+     * @return a `Parser' that -- on success -- returns the result of `q'.
+     */
     def ~> [U](p: => Parser[U]): Parser[U] = (for(a <- this; b <- p) yield b).named("~>")
+
+    /** A parser combinator for sequential composition which keeps only the left result 
+     *
+     * <p> `p &lt;~ q' succeeds if `p' succeeds and `q' succeeds on the input
+     *           left over by `p'.</p>
+     * 
+     * <b>Note:</b> &lt;~ has lower operator precedence than ~ or ~>.
+     *
+     * @param q a parser that will be executed after `p' (this parser) succeeds
+     * @return a `Parser' that -- on success -- returns the result of `p'.
+     */
     def <~ [U](p: => Parser[U]): Parser[T] = (for(a <- this; b <- p) yield a).named("<~")
 
-   
      /* not really useful: V cannot be inferred because Parser is covariant in first type parameter (V is always trivially Nothing)
     def ~~ [U, V](q: => Parser[U])(implicit combine: (T, U) => V): Parser[V] = new Parser[V] {
       def apply(in: Input) = seq(Parser.this, q)((x, y) => combine(x,y))(in)
@@ -275,7 +288,7 @@ trait Parsers {
      *          characters accepts.</p>
      * 
      * @param q a parser that accepts if p consumes less characters.
-     * @return a `Parser' that returns the result of the parser consuming the most characteres (out of `p' and `q').
+     * @return a `Parser' that returns the result of the parser consuming the most characters (out of `p' and `q').
      */
     def ||| [U >: T](q: => Parser[U]): Parser[U] = new Parser[U] {
       def apply(in: Input) = {
@@ -287,8 +300,7 @@ trait Parsers {
           case (s1 @ Success(_, _), _) => s1
           case (_, s2 @ Success(_, _)) => s2
           case (e1 @ Error(_, _), _) => e1
-          case (f1 @ Failure(_, next1), f2 @ Failure(_, next2)) => if (next2.pos < next1.pos) f1 else f2
-          case (f1 @ Failure(_, next1), e2 @ Error(_, next2)) => if (next2.pos < next1.pos) f1 else e2
+          case (f1 @ Failure(_, next1), ns2 @ NoSuccess(_, next2)) => if (next2.pos < next1.pos) f1 else ns2
         }
       }
       override def toString = "|||"
@@ -336,7 +348,7 @@ trait Parsers {
     def ^? [U](f: PartialFunction[T, U]): Parser[U] = ^?(f, r => "Constructor function not defined at "+r)
     
        
-    /** A parser combinator that parameterises a subsequent parser with the result of this one
+    /** A parser combinator that parameterizes a subsequent parser with the result of this one
      *
      *<p>
      * Use this combinator when a parser depends on the result of a previous parser. `p' should be
@@ -512,8 +524,7 @@ trait Parsers {
     val r = p(in)
     println(name +" --> "+ r)
     r
-  } 
-
+  }
   
   /** A parser generator for repetitions.
    *  
@@ -564,25 +575,24 @@ trait Parsers {
    * @return A parser that returns a list of results produced by first applying `f' and then 
    *         repeatedly `p' to the input (it only succeeds if `f' matches).
    */
-  def rep1[T](first: => Parser[T], p: => Parser[T]): Parser[List[T]] = first ~ rep(p) ^^ { case ~(x, xs) => x :: xs }
-  
-  /* new Parser[List[T]] {
-    def apply(in0: Input) = {
-      val xs = new scala.collection.mutable.ListBuffer[T]
-      var in = in0
-      
-      var res = first(in)
-      
-      while(res.successful) {
-        xs += res.get
-        in = res.next
-        res = p(in)
+  def rep1[T](first: => Parser[T], p: => Parser[T]): Parser[List[T]] = Parser { in =>
+    val elems = new ListBuffer[T]
+
+    def continue(in: Input): ParseResult[List[T]] = {
+      val p0 = p    // avoid repeatedly re-evaluating by-name parser
+      @tailrec def applyp(in0: Input): ParseResult[List[T]] = p0(in0) match {
+        case Success(x, rest) => elems += x ; applyp(rest)
+        case _                => Success(elems.toList, in0)
       }
       
-      if (!xs.isEmpty) Success(xs.toList, res.next)
-      else Failure("TODO", TODO)
+      applyp(in)
     }
-  }*/
+
+    first(in) match {
+      case Success(x, rest) => elems += x ; continue(rest)
+      case ns: NoSuccess    => ns
+    }
+  }
 
   /** A parser generator for a specified number of repetitions.
    *  
@@ -594,16 +604,26 @@ trait Parsers {
    * @return A parser that returns a list of results produced by repeatedly applying `p' to the input
    *        (and that only succeeds if `p' matches exactly `n' times).
    */  
-  def repN[T](n: Int, p: => Parser[T]): Parser[List[T]] = 
-    if(n==0) success(Nil) else p ~ repN(n-1, p) ^^ { case ~(x, xs) => x :: xs }
+  def repN[T](num: Int, p: => Parser[T]): Parser[List[T]] = 
+    if (num == 0) success(Nil) else Parser { in =>
+      val elems = new ListBuffer[T]
+      val p0 = p    // avoid repeatedly re-evaluating by-name parser
+      
+      @tailrec def applyp(in0: Input): ParseResult[List[T]] =
+        if (elems.length == num) Success(elems.toList, in0)
+        else p0(in0) match {
+          case Success(x, rest)   => elems += x ; applyp(rest)
+          case ns: NoSuccess      => return ns
+        }
+      
+      applyp(in)
+    }
   
   /** A parser generator for non-empty repetitions.
    *  
-   *  <p>rep1sep(first, p, q) starts by using `first', followed by repeatedly uses of `p' interleaved with `q' 
-   *                to parse the input, until `p' fails. `first' must succeed (the result is a `List' of the 
-   *                consecutive results of `first' and `p')</p>
+   *  <p>rep1sep(p, q) repeatedly applies `p' interleaved with `q' to parse the input, until `p' fails.
+   *                The parser `p' must succeed at least once.</p>
    *
-   * @param first a `Parser' that is to be applied to the first element of input
    * @param p a `Parser' that is to be applied successively to the input
    * @param q a `Parser' that parses the elements that separate the elements parsed by `p' 
    *          (interleaved with `q')   
@@ -611,9 +631,10 @@ trait Parsers {
    *         (and that only succeeds if `p' matches at least once).
    *         The results of `p' are collected in a list. The results of `q' are discarded.    
    */
-  def rep1sep[T](p: => Parser[T], q: => Parser[Any]): Parser[List[T]] = 
-    p ~ (q ~ rep1sep(p, q) ^^ { case x ~ y => y } | success(List())) ^^ { case x ~ y => x :: y }
+  def rep1sep[T](p : => Parser[T], q : => Parser[Any]): Parser[List[T]] = 
+    p ~ rep(q ~> p) ^^ {case x~y => x::y} 
    
+
   /** A parser generator that, roughly, generalises the rep1sep generator so that `q', which parses the separator,
    * produces a left-associative function that combines the elements it separates.
    *
@@ -641,7 +662,7 @@ trait Parsers {
         
   /** A parser generator that generalises the rep1sep generator so that `q', which parses the separator,
    * produces a right-associative function that combines the elements it separates. Additionally,
-   * The right-most (last) element and the left-most combinating function have to be supplied.
+   * The right-most (last) element and the left-most combining function have to be supplied.
    * 
    * rep1sep(p: Parser[T], q) corresponds to chainr1(p, q ^^ cons, cons, Nil) (where val cons = (x: T, y: List[T]) => x :: y)
    *
@@ -672,12 +693,24 @@ trait Parsers {
    */
   def not[T](p: => Parser[T]): Parser[Unit] = Parser { in =>
     p(in) match {
-      case s @ Success(_, _) => Failure("Expected failure", in)
-      case e @ Error(_, _) => Success((), in)
-      case f @ Failure(msg, next) => Success((), in)
+      case Success(_, _)  => Failure("Expected failure", in)
+      case _              => Success((), in)
     }
   }  
-  
+
+  /** A parser generator for guard expressions. The resulting parser will fail or succeed 
+   * just like the one given as parameter but it will not consume any input.
+   *
+   * @param p a `Parser' that is to be applied to the input
+   * @return A parser that returns success if and only if 'p' succeeds but never consumes any input
+   */
+  def guard[T](p: => Parser[T]): Parser[T] = Parser { in =>
+    p(in) match{
+      case s@ Success(s1,_) => Success(s1, in)
+      case e => e
+    }
+  }
+
   
   /** `positioned' decorates a parser's result with the start position of the input it consumed. 
    * 
@@ -708,15 +741,20 @@ trait Parsers {
   def phrase[T](p: Parser[T]) = new Parser[T] {
     lastNoSuccess = null
     def apply(in: Input) = p(in) match {
-      case s @ Success(out, in1) if in1.atEnd => s
-      case s @ Success(out, in1) => Failure("end of input expected", in1)
-      case f : NoSuccess => lastNoSuccess
+      case s @ Success(out, in1) =>
+        if (in1.atEnd) 
+          s
+        else if (lastNoSuccess == null || lastNoSuccess.next.pos < in1.pos)
+          Failure("end of input expected", in1)
+        else 
+          lastNoSuccess
+      case _ => lastNoSuccess
     }
   }
 
   def mkList[T] = (_: ~[T, List[T]]) match { case x ~ xs => x :: xs }
   case class ~[+a, +b](_1: a, _2: b) {
-    override def toString = "("+ _1 +" ~ "+ _2 +")"
+    override def toString = "("+ _1 +"~"+ _2 +")"
   }
 
   /** A parser whose ~ combinator disallows back-tracking.
